import csv
import CrawlUtils
import sys
from collections import Counter
from collections import defaultdict
"""
Finds network connections of datasets 
1. known to us (as we otherwise might not know if the target actually holds linked data)
with
2. predicates that we deemed to be used for linking to other resources

Parameters:
Input-Related
inputFileName = The nq file with candidate triples
categoryFile= categories of datasets
mode="sameAs" or something else, were for sameAs only predicate "sameAs" is considered

discardingPropertiesFile			contains predicates not used for linking aka negative predicate list
knownConnectingEntitiesFilesName	properties that are known to connect entities. aka positive predicate list

Output-Related
edgeFile=lists all edges
predicateFile= lists all predicates and their degree i.e. the number of datasets used that use them for outgoing/ingoing links
degreePredicateFile=datasets and their in/outdegree for every predicate
vocabFile=vocabs and their degree


"""

discardingProperties = set()
knownProperties = set()
knownGraphs = set()
rejectGraphs = set()
categories = {}

nodeDegrees = {}
nodeDegreesWithPredicates = {}
arcs = {}
arcsWithPredicates = {}
predicateUsage = {}
vocabUsage = {}
noExternalLinks = {}
externalLinks = {}
resourceInDegree = defaultdict(Counter)


def getConnections(inputFileName, categoryFile, edgeOutputFile, degreeOutputFile, edgePredicateOutputFile, degreePredicateFile, predicateFile, linksetFile, vocabFile, discardingPropertiesFile,knownConnectingEntitiesFilesName, linkedResourcesTopFileName):
    
    #Reading Categories of datasets
    with open(categoryFile) as inputFile:
		
        csvreader = csv.reader(inputFile, delimiter='\t')
        
        for row in csvreader:
			
            dataset = row[0]
            categories[dataset] = row[1]
            knownGraphs.add(dataset)
            nodeDegrees[dataset] = [set(),set()]
            nodeDegreesWithPredicates[dataset] = {}
            externalLinks[dataset] = Counter()
            
    inputFile.close
    
    #Reading file with properties that are not used for interlinking
    with open(discardingPropertiesFile) as inputFile:
		
        for line in inputFile:
			
            discardingProperties.add(line.rstrip('\n').lower())
            
    inputFile.close
    
    #Read Predicates that are known to connect entities
    with open(knownConnectingEntitiesFilesName) as inputFile:
		
        for line in inputFile:
			
            knownProperties.add(line.rstrip('\n').lower())
            
    inputFile.close
     
    csv.field_size_limit(sys.maxsize)
    with open(inputFileName) as inputFile:
		
        lineNo = 0
        csvreader = csv.reader(inputFile, delimiter='\t')
        
        for row in csvreader:

            graph = CrawlUtils.getDataset(row[0])
            lineNo += 1
            
            if lineNo == 1:
                
                continue
            
            if lineNo % 100000 == 0:
                
                print(lineNo)
            
            if row[2].lower() in discardingProperties or not row[2].lower() in knownProperties:
				
                continue
            
            if graph not in knownGraphs:
				
                continue
            
            if row[1][:2] == "_:" and row[3][:2] == "_:":
				
                continue
            
            if row[1][:2] == "_:":
				
                subjectDataset = graph
            
            else:
				
                subjectDataset = CrawlUtils.getDataset(row[1])
                
            if row[3][:2] == "_:":
				
                objectDataset = graph
                
            else:
				
                objectDataset = CrawlUtils.getDataset(row[3])
                
            hasExternalLink=0
            connectingPredicate = row[2].lower()
            
            if not (subjectDataset != None and objectDataset != None):
				
                continue
                
            if subjectDataset == graph:
				
                startDataset = subjectDataset
                destinationDataset = objectDataset
                
            elif objectDataset == graph:
				
                startDataset = objectDataset
                destinationDataset = subjectDataset
                
            elif subjectDataset != graph and objectDataset != graph and subjectDataset != objectDataset and row[1][:2] != "_:" and row[3][:2] != "_:":
				
                startDataset = subjectDataset
                destinationDataset = objectDataset
                externalLinks[graph] = startDataset+"\t"+destinationDataset
                hasExternalLink = 1
            
            if startDataset in knownGraphs and destinationDataset in knownGraphs:
                
                if startDataset != destinationDataset:

                    if objectDataset == graph and row[3][:2] != '_:' :
						
                        resourceInDegree[graph][row[3]] += 1
                        
                    nodeDegrees[startDataset][0].add(destinationDataset)
                    nodeDegrees[destinationDataset][1].add(startDataset)
                    
                    startDatasetPredicates = nodeDegreesWithPredicates.get(startDataset)
                    startDatasetPredicatesSet = startDatasetPredicates.get(connectingPredicate, [set(),set()])
                    startDatasetPredicatesSet[0].add(destinationDataset)
                    startDatasetPredicates[connectingPredicate] = startDatasetPredicatesSet
                    nodeDegreesWithPredicates[startDataset] = startDatasetPredicates
                    
                    destinationDatasetPredicates= nodeDegreesWithPredicates.get(destinationDataset)
                    destinationDatasetPredicateSet = destinationDatasetPredicates.get(connectingPredicate, [set(),set()])
                    destinationDatasetPredicateSet[1].add(startDataset)
                    destinationDatasetPredicates[connectingPredicate] = destinationDatasetPredicateSet
                    nodeDegreesWithPredicates[destinationDataset] = destinationDatasetPredicates
                
                    targetNodes = arcs.get(startDataset, {})
                    
                    if targetNodes.get(destinationDataset) == None:
						
                        targetNodes[destinationDataset] = 0
                        
                    targetNodes[destinationDataset] += 1
                    arcs[startDataset] = targetNodes
                    
                    #Update EdgesWithPredicates
                    targetNodes = arcsWithPredicates.get(startDataset, {})
                    connectionPredicates = targetNodes.get(destinationDataset, {})
                    connectionPredicates[connectingPredicate] = connectionPredicates.get(connectingPredicate, 0) + 1
                    targetNodes[destinationDataset] = connectionPredicates
                    arcsWithPredicates[startDataset] = targetNodes
                    
                    #Update Predicate Usage
                    predicateStats = predicateUsage.get(connectingPredicate,[set(), set(), 0])
                    predicateStats[0].add(startDataset)
                    predicateStats[1].add(destinationDataset)
                    predicateStats[2] += 1
                   
                    
                    predicateUsage[connectingPredicate] = predicateStats
                    
                    #Update Vocab Usage
                    connectingvocab = CrawlUtils.getVocabFromTerm(connectingPredicate)
                    vocabStats = vocabUsage.get(connectingvocab,[set(), set(), 0])
                    vocabStats[0].add(startDataset)
                    vocabStats[1].add(destinationDataset)
                    vocabStats[2] += 1
                    
                    #vocabStats = [vocabStats[0].add(startDataset),vocabStats[1].add(destinationDataset), vocabStats[2]+1]
                    vocabUsage[connectingvocab] = vocabStats
                    
                
                
    inputFile.close
   
    print("Writing results")
    
    with open(edgeOutputFile, 'w') as outputFileEdges:
		
        outputFileEdges.write('startDataset\tcategory\tendDataset\tcategory\tlinks\n')
        
        for startNode, endNodes in arcs.items():
			
            for endNode, links in endNodes.items():
				
                outputFileEdges.write('\t'.join([startNode, categories.get(startNode), endNode, categories.get(endNode), str(links)])+'\n')
                
    outputFileEdges.close
    
    with open(edgePredicateOutputFile, 'w') as outputFileEdgesPredicates:
		
            outputFileEdgesPredicates.write('startDataset\tcategory\tendDataset\tcategory\tlinkingPredicate\tlinks\n')
            
            for startNode, endNodes in arcsWithPredicates.items():
				
                for endNode, predicates in endNodes.items():
					
                    for predicate, links in predicates.items():
						
                        outputFileEdgesPredicates.write('\t'.join([startNode, categories.get(startNode), endNode, categories.get(endNode), predicate, str(links)])+'\n')
                        
    outputFileEdgesPredicates.close
    
    with open(degreeOutputFile, 'w') as outputFileDegree:
		
        outputFileDegree.write("dataset\tcategory\toutdegree\tindegree\n")
        
        for dataset, degrees in nodeDegrees.items():
			
            outputFileDegree.write('\t'.join([dataset,categories.get(dataset),str(len(degrees[0])),str(len(degrees[1]))])+'\n')
            
    outputFileDegree.close
    
    with open(degreePredicateFile, 'w') as outputFileDegreePredicates:
		
        outputFileDegreePredicates.write("dataset\tcategory\tpredicate\toutdegree\tindegree\n")
        
        for dataset, predicates in nodeDegreesWithPredicates.items():
			
            for predicate, degrees in predicates.items():
				
                outputFileDegreePredicates.write('\t'.join([dataset,categories.get(dataset),predicate,str(len(degrees[0])),str(len(degrees[1]))])+'\n')
                
    outputFileDegreePredicates.close
    
    with open(predicateFile, 'w') as outputPredicateFile:
		
        outputPredicateFile.write('Predicate\tInDegree\tOutDegree\tTriples\n')
        
        for predicate, stats in predicateUsage.items():
			
            outputPredicateFile.write('\t'.join([predicate,str(len(stats[0])),str(len(stats[1])),str(stats[2])])+'\n')
            
    outputPredicateFile.close
    
    with open(vocabFile, 'w') as outputVocabFile:
		
        outputVocabFile.write('Vocab\tInDegree\tOutDegree\tTriples\n')
        
        for vocab, stats in vocabUsage.items():
			
            outputVocabFile.write('\t'.join([vocab,str(len(stats[0])),str(len(stats[1])),str(stats[2])])+'\n')
            
    outputVocabFile.close
    
    with open(linksetFile, 'w') as outputLinksetFile:
		
        outputLinksetFile.write('graph\tingraph\toutgraph\tTriples\n')
        
        for graph, linkCombination in externalLinks.items():
			
            outputLinksetFile.write(graph+"\t"+str(linkCombination)+'\n')
            
    outputLinksetFile.close
    
    
    with open(linkedResourcesTopFileName, 'w') as outputFile:
		
        for graph, resources in resourceInDegree.items():
			
            sorted_resources = sorted(resources.items(), key=lambda x:[1],reverse=True)
            
            outputFile.write(graph+'\t'+str(sorted_resources[0][1])+'\t'+str(sorted_resources[0][2])+'\n')
            
    outputFile.close
    
    print("Done")
if __name__ == "__main__":
	
    CrawlUtils.initializeDatasetList()
    getConnections(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4], sys.argv[5], sys.argv[6],sys.argv[7],sys.argv[8],sys.argv[9],sys.argv[10], sys.argv[11], sys.argv[12])
                
                
            
            
            
