import csv
import sys
import CrawlUtils

        
PLDVocabUsage = {}
vocabUsageByPLDCount = {}
vocabTerms = {}
datasetList = set()
dctNamespaces = set(["http://purl.org/dc/elements/1.1/","http://purl.org/dc/terms/","http://dublincore.org/documents/dces/"])

def getVocabLists(graphterms, categoryFileName,PLDVocabUsageFileName,vocabUsageByPLDCountFileName, vocabTermsFileName, mode):
	
    with open(categoryFileName) as inputFile:
		
        csvreader = csv.reader(inputFile, delimiter='\t')
        for row in csvreader:
            datasetList.add(row[0])
            
    inputFile.close
    
    print ("Datasets used:"+str(len(datasetList)))
    
    prefixes={}
    
    with open('./prefixes.tsv') as prefixFile:
		
        csvreader= csv.reader(prefixFile, delimiter='\t')
        
        for prefix,vocab in csvreader:
			
            prefixes[vocab]=prefix
            
    prefixFile.close()
    
    with open(graphterms) as inputFile:
		
        print("Opening "+graphterms)
        
        csvreader= csv.reader(inputFile, delimiter='\t')
        index = 0
        
        for row in csvreader:
            index += 1
            if index == 1:
                continue
                
            dataset = row[0]
            term = row[1]
            frequency = int(row[2])
            if dataset not in datasetList:
				
                continue
            if mode == "PLD":
				
                vocab = CrawlUtils.getVocabPLDFromTerm(term)
                #Replaces variants of DC/DCT with DCT-NS
                if vocab in dctNamespaces:
                    vocab = "http://purl.org/dc/terms/"
                    
            else:
                vocab = CrawlUtils.getVocabFromTerm(term)
                
            if vocab == None or vocab == "http://":
				
                continue
                
            vocabTermSet = vocabTerms.get(vocab)
            
            if vocabTermSet == None:
				
                vocabTermSet = set()
                
            vocabTermSet.add(term)
            vocabTerms[vocab] = vocabTermSet
            
            vocabDict = PLDVocabUsage.get(dataset)
            
            if vocabDict == None:
				
                vocabDict = {}
                
            links = vocabDict.get(vocab)
            
            if links == None:
				
                links = 0
                
            links += frequency
            vocabDict[vocab] = links
            PLDVocabUsage[dataset]= vocabDict
            usingPLDs = vocabUsageByPLDCount.get(vocab)
            
            if usingPLDs == None:
				
                usingPLDs = set()
                
            usingPLDs.add(dataset)
            vocabUsageByPLDCount[vocab] = usingPLDs
            
    inputFile.close()
    
    vocabUsageByPLDCountWithAggregation = {}
    
    for vocab, PLDSet in vocabUsageByPLDCount.items():
		
        if len(PLDSet) > 1:
			
            vocabUsageByPLDCountWithAggregation[vocab] = PLDSet
            
        else:
			
            vocabPLD = CrawlUtils.getPLD(vocab)
            usingPLDs = vocabUsageByPLDCountWithAggregation.get(vocabPLD)
            
            if usingPLDs == None:
				
                usingPLDs = set()
                
            usingPLDs = usingPLDs.union(PLDSet)
            vocabUsageByPLDCountWithAggregation[vocabPLD] = usingPLDs
        
    
    
    with open(PLDVocabUsageFileName,'w') as PLDOutputFile:
        
        for dataset, vocabList in PLDVocabUsage.items():
			
            noTriples = 0
            
            for frequency in vocabList.values():
				
                noTriples += frequency
                
            for vocab, frequency in vocabList.items():
				
                if vocab == None:
					
                    continue
                    
                PLDOutputFile.write(dataset+'\t'+vocab+'\t'+str(frequency)+'\t'+str(frequency/noTriples * 100)+'\n')
                
    PLDOutputFile.close()
    
   
    print("Total number of PLDs: "+str(len(PLDVocabUsage.keys())))
    
    with open(vocabUsageByPLDCountFileName,'w') as VocabOutputFile:
		
        UsingNonPropVocabs = set()
        UsingPropVocabs = set()
        totalPLDNo=len(PLDVocabUsage.keys())
        for vocab, usingPLDs in vocabUsageByPLDCount.items():
            UsingPLDs = len(usingPLDs)
            if not (vocab == "http://www.w3.org/1999/02/22-rdf-syntax-ns#" or vocab == "http://www.w3.org/2000/01/rdf-schema#" or vocab == "http://www.w3.org/2002/07/owl#" or vocab == "http://www.w3.org/2006/http#" or vocab == "http://code.google.com/p/ldspider/ns#"):
				
                if UsingPLDs > 1:
					
                    UsingNonPropVocabs = UsingNonPropVocabs.union(usingPLDs)
                    #print("Adding "+vocab+" with "+str(len(usingPLDs)) +" leading to "+str(len(UsingNonPropVocabs)))
                    
                else :
					
                    UsingPropVocabs = UsingPropVocabs.union(usingPLDs)
                    
            else :
				
                print (vocab+" with "+str(len(usingPLDs)))
                
            prefix = prefixes.get(vocab)
            
            if prefix == None:
				
                prefix = ""
                
            VocabOutputFile.write(prefix+'\t'+vocab+'\t'+str(UsingPLDs)+'\t'+str(UsingPLDs/totalPLDNo)+'\n')
            
        print('UsingNonProprietaryVocabs'+'\t'+str(len(UsingNonPropVocabs)))
        
        for ds in datasetList.difference(UsingNonPropVocabs):
			
            print(str(ds))
            
        print('UsingPropVocabs'+'\t'+str(len(UsingPropVocabs)))

        
    VocabOutputFile.close()            
    
    with open(vocabTermsFileName, 'w') as VocabTermOutputFile:
		
        for vocab, termset in vocabTerms.items():
			
            for term in termset:
				
                VocabTermOutputFile.write(vocab+'\t'+term+'\n')
                
    VocabTermOutputFile.close

if __name__ == "__main__":
	
    getVocabLists(sys.argv[1], sys.argv[2],sys.argv[3],sys.argv[4], sys.argv[5],sys.argv[6])
